%This m-file is used to calculate the term structure and the resulting expectations of the development of the exchange rate
%These results are used to calculate the one period depreciation rates. These are compared to the one period depreciation
%rates following from the expected future one period total return differences
%The values of the variables need to be set in advance, e.g. using characterize.m
clear all;

load ER_X_Y_at_t.mat
%Current exchange rate
ER_X_Y=ER_X_Y_at_t(3);
%Current time
t=ER_X_Y_at_t(1);
lasttime=ER_X_Y_at_t(2);

%COUNTRY X
%COUNTRY X with following values
load ValX.mat
r_X=ValX(1);
kappa=ValX(2);
theta=ValX(3);
sigma=ValX(4);
lambda=ValX(5);
gamma=((kappa+lambda)^2+2*(sigma^2))^0.5;
%Long term value of the yield
Rl_X=(2*kappa*theta)/(gamma+kappa+lambda)
%Critical value, which determines whether or not the term structure is humped
Rg_X=(kappa*theta)/(kappa+lambda)
phi1=gamma;
phi2=(kappa+lambda+gamma)*0.5;
phi3=(2*kappa*theta)/(sigma^2);

%Calculation of the term structure of COUNTRY X
%Yield converges to the spot rate as maturity nears: Tau==Null
Rrt(1)=r_X;
%Value of zero bond at maturity and effective interest rate
Prt(1)=1;
i=1;    
for Tau=0.01:0.01:lasttime-t
    i=i+1;
    N=phi2*(exp(phi1*(Tau))-1)+phi1;    
    At=(phi1*exp(phi2*(Tau))/N)^phi3;
    Bt=(exp(phi1*(Tau))-1)/N;
    Prt(i)=At*exp(-Bt*r_X);
    if At~=0
        Rrt(i)=(-log(At)+r_X*Bt)/(Tau);
    else
        Rrt(i)=NaN;
    end
end

N=phi2*(exp(phi1*(0.01))-1)+phi1;    
At=(phi1*exp(phi2*(0.01))/N)^phi3;
Bt=(exp(phi1*(0.01))-1)/N;
%Value in the long-run
Yl=(1/At)*((2*kappa/sigma^2)/((2*kappa/sigma^2)-Bt))^(2*kappa*theta/sigma^2)-1
%The expected total return for a zero bond with maturity=0.01 at current time t
%can be calculated as follows:
RTRX(1)=exp(Rrt(2)*0.01)-1;
i=1;
for s=0.01+t:0.01:lasttime-0.01
    i=i+1;    
    c=(2.*kappa)./(sigma.^2.*(1-exp(-kappa.*(s-t))));
    u=c.*r_X.*exp(-kappa.*(s-t));
    q=(2.*kappa.*theta)./(sigma.^2)-1;
    RTRX(i)=(1/At)*(c/(c-Bt))^(q+1)*exp((u*Bt)/(c-Bt))-1;
end

%Saving
ValuesX=[t:0.01:lasttime;Rrt;Prt]; 
datafile=['X',mat2str(r_X),'_',mat2str(kappa),'_',mat2str(theta),'_',mat2str(sigma),'_',mat2str(lambda),'.mat'];
save(datafile,'ValuesX');
phi_rX=[phi1,phi2,phi3,r_X,kappa,theta,sigma,lambda,gamma];
save X.mat phi_rX;

clear kappa theta sigma lambda gamma phi1 phi2 phi3 i Tau Rrt Prt r_X s c u q;

%COUNTRY Y 
%COUNTRY Y with following values
load ValY.mat
r_Y=ValY(1);
kappa=ValY(2);
theta=ValY(3);
sigma=ValY(4);
lambda=ValY(5);
gamma=((kappa+lambda)^2+2*(sigma^2))^0.5;
%Long term value of the yield
Rl_Y=(2*kappa*theta)/(gamma+kappa+lambda)
%Critical value, which determines whether or not the term structure is humped
Rg_Y=(kappa*theta)/(kappa+lambda)
phi1=gamma;
phi2=(kappa+lambda+gamma)*0.5;
phi3=(2*kappa*theta)/(sigma^2);

%Calculation of the term structure of COUNTRY Y
%Yield converges to the spot rate as maturity nears: Tau==Null
Rrt(1)=r_Y;
%Value of zero bond at maturity and effective yield
Prt(1)=1;
i=1;    
for Tau=0.01:0.01:lasttime-t
    i=i+1;
    N=phi2*(exp(phi1*(Tau))-1)+phi1;    
    At=(phi1*exp(phi2*(Tau))/N)^phi3;
    Bt=(exp(phi1*(Tau))-1)/N;
    Prt(i)=At*exp(-Bt*r_Y);
    if At~=0
        Rrt(i)=(-log(At)+r_Y*Bt)/(Tau);
    else
        Rrt(i)=NaN;
    end
end

N=phi2*(exp(phi1*(0.01))-1)+phi1;    
At=(phi1*exp(phi2*(0.01))/N)^phi3;
Bt=(exp(phi1*(0.01))-1)/N;
%Value in the long-run
Yl=(1/At)*((2*kappa/sigma^2)/((2*kappa/sigma^2)-Bt))^(2*kappa*theta/sigma^2)-1
%The expected total return for a zero bond with maturity=0.01 at time current time t 
%can be calculated as follows:
RTRY(1)=exp(Rrt(2)*0.01)-1;
i=1;
for s=0.01+t:0.01:lasttime-0.01
    i=i+1;    
    c=(2.*kappa)./(sigma.^2.*(1-exp(-kappa.*(s-t))));
    u=c.*r_Y.*exp(-kappa.*(s-t));
    q=(2.*kappa.*theta)./(sigma.^2)-1;
    RTRY(i)=(1/At)*(c/(c-Bt))^(q+1)*exp((u*Bt)/(c-Bt))-1;
end

%Saving%
ValuesY=[t:0.01:lasttime;Rrt;Prt]; 
datafile=['Y',mat2str(r_Y),'_',mat2str(kappa),'_',mat2str(theta),'_',mat2str(sigma),'_',mat2str(lambda),'.mat'];
save(datafile,'ValuesY');
phi_rY=[phi1,phi2,phi3,r_Y,kappa,theta,sigma,lambda,gamma];
save Y.mat phi_rY;

clear kappa theta sigma lambda gamma phi1 phi2 phi3 i Tau Rrt Prt r_Y s c u q;

%The expected exchange rates
%At current time the expectation equals the spot exchange rate
ExER(1)=ER_X_Y;
i=1;
for j=0.01+t:0.01:lasttime
    i=i+1;
    ExER(i)=((1/ValuesX(3,i))/(1/ValuesY(3,i)))*ER_X_Y;
end
save ExER.mat ExER;

%Here the percentage change of the exchange rate from one period to another is calculated
%The values of the expected exchange rate are used
%f(i+1) defines the forward rate: i->i+1
i=0;
for j=0.01+t:0.01:lasttime
    i=i+1;
    f(i+1)=(ExER(i+1)-ExER(i))/ExER(i);
end

%Here the percentage change of the exchange rate from one period to another is calculated
%The values of the differences of expected future one period total returns are used
l=length(RTRY);
TRdiff=[RTRX(1:l)-RTRY(1:l)]./(1+RTRY(1:l));
l=length(f);

%Plot
visu1=figure;
plot(ValuesX(1,:),ValuesX(2,:),'r',ValuesY(1,:),ValuesY(2,:));axis tight
title(['X: ',texlabel('r='),mat2str(phi_rX(4)),';',texlabel('kappa='),mat2str(phi_rX(5)),';',texlabel('theta='),mat2str(phi_rX(6)),';',texlabel('sigma='),mat2str(phi_rX(7)),';',texlabel('lambda='),mat2str(phi_rX(8)),'   Y: ',texlabel('r='),mat2str(phi_rY(4)),';',texlabel('kappa='),mat2str(phi_rY(5)),';',texlabel('theta='),mat2str(phi_rY(6)),';',texlabel('sigma='),mat2str(phi_rY(7)),';',texlabel('lambda='),mat2str(phi_rY(8))],'Fontsize', 11);legend('X','Y',0);xlabel('Time-to-maturity');ylabel('Yield-to-maturity');grid on;
visu2=figure;
plot(0+t:0.01:lasttime,ExER,'m');xlabel('Time');ylabel('Expected Exchange Rate');axis tight
title(['X: ',texlabel('r='),mat2str(phi_rX(4)),';',texlabel('kappa='),mat2str(phi_rX(5)),';',texlabel('theta='),mat2str(phi_rX(6)),';',texlabel('sigma='),mat2str(phi_rX(7)),';',texlabel('lambda='),mat2str(phi_rX(8)),'   Y: ',texlabel('r='),mat2str(phi_rY(4)),';',texlabel('kappa='),mat2str(phi_rY(5)),';',texlabel('theta='),mat2str(phi_rY(6)),';',texlabel('sigma='),mat2str(phi_rY(7)),';',texlabel('lambda='),mat2str(phi_rY(8))],'Fontsize', 11);grid on;
visu3=figure;
plot(0.01+t:0.01:lasttime,f(2:l),'m');hold on;plot(0.01+t:0.01:lasttime,TRdiff,'g');grid on;
title(['X: ',texlabel('r='),mat2str(phi_rX(4)),';',texlabel('kappa='),mat2str(phi_rX(5)),';',texlabel('theta='),mat2str(phi_rX(6)),';',texlabel('sigma='),mat2str(phi_rX(7)),';',texlabel('lambda='),mat2str(phi_rX(8)),'   Y: ',texlabel('r='),mat2str(phi_rY(4)),';',texlabel('kappa='),mat2str(phi_rY(5)),';',texlabel('theta='),mat2str(phi_rY(6)),';',texlabel('sigma='),mat2str(phi_rY(7)),';',texlabel('lambda='),mat2str(phi_rY(8))],'Fontsize', 11);grid on;
legend('based on term structure','based on DiffExpTR(0.01)',0);xlabel('Time');ylabel('Expected depreciation rate');

%SAVING OF PLOT with counting number i
i=9;
saveas(visu1,['Exrates',mat2str(i),'-1.fig']);
saveas(visu2,['Exrates',mat2str(i),'-2.fig']);
saveas(visu3,['Exrates',mat2str(i),'-3.fig']);
saveas(visu1,['Exrates',mat2str(i),'-1.eps']);
saveas(visu2,['Exrates',mat2str(i),'-2.eps']);
saveas(visu3,['Exrates',mat2str(i),'-3.eps']);

clear all;