%This m-file is used to test whether the investment in bonds with 
%longer maturity pays off more than the repeated investment in bonds with maturity=0.01
clear all;

load ER_X_Y_at_t.mat
%Current exchange rate
ER_X_Y=ER_X_Y_at_t(3);
%Current time
t=ER_X_Y_at_t(1);
lasttime=ER_X_Y_at_t(2);

%COUNTRY X 
%COUNTRY X with following values
load ValX.mat
r=ValX(1);
kappa=ValX(2);
theta=ValX(3);
sigma=ValX(4);
lambda=ValX(5);
gamma=((kappa+lambda)^2+2*(sigma^2))^0.5;
%Long term value of the yield%
Rl=(2*kappa*theta)/(gamma+kappa+lambda);
%Critical value, which determines whether or not the term structure is humped
Rg=(kappa*theta)/(kappa+lambda);
phi1=gamma;
phi2=(kappa+lambda+gamma)*0.5;
phi3=(2*kappa*theta)/(sigma^2);

%This part calculates the term structure of default-free discount bonds accoding to the paper of CIRII
%Yield converges to the spot rate as maturity nears: Tau==Null
Rrt(1)=r;
i=1;    
for Tau=0.01:0.01:lasttime-t
    i=i+1;
    N=phi2*(exp(phi1*(Tau))-1)+phi1;    
    At=(phi1*exp(phi2*(Tau))/N)^phi3;
    Bt=(exp(phi1*(Tau))-1)/N;
    if At~=0
        Rrt(i)=(-log(At)+r*Bt)/(Tau);
        RTRl(i-1)=exp(Rrt(i)*Tau)-1;
    else
        Rrt(i)=NaN;
        RTRl(i-1)=NaN;
    end
end

%Plotting the effective yield when at time t=0 a zero bond with maturity Tau=[0.01:0.01:lasttime-t] is chosen
visu5=figure;
Values=[0.01+t:0.01:lasttime;RTRl]; 
plot(Values(1,:),Values(2,:));hold on;axis tight

%These values can be chosen constant, cause we only investigate bonds with maturity=0.01
N=phi2*(exp(phi1*(0.01))-1)+phi1;    
At=(phi1*exp(phi2*(0.01))/N)^phi3;
Bt=(exp(phi1*(0.01))-1)/N;

%This part calculates the expectation of the total return when reinvesting every period
%The expected total return for a zero bond with maturity=0.01 at current time t
%can be calculated as follows:
RTR(1)=exp(Rrt(2)*0.01)-1;
i=1;
for s=0.01+t:0.01:lasttime-0.01
    i=i+1;    
    c=(2.*kappa)./(sigma.^2.*(1-exp(-kappa.*(s-t))));
    u=c.*r.*exp(-kappa.*(s-t));
    q=(2.*kappa.*theta)./(sigma.^2)-1;
    RTR(i)=(1/At)*(c/(c-Bt))^(q+1)*exp((u*Bt)/(c-Bt))-1;
end

%Calculation of the total return when reinvesting every period
i=1;
for s=0.01+t:0.01:lasttime    
    QMRTR(i)=1;
    for j=1:i
        QMRTR(i)=QMRTR(i)*(RTR(j)+1);
    end
    QMRTR(i)=QMRTR(i)-1;
    i=i+1;
end

%Plotting of the total return when reinvesting every period
plot(Values(1,:),QMRTR,'r');hold off;axis tight
title('Country X: Long-term bonds vs. repeated reinvestment');
legend('Total return of long-term bond','Expected total return of repeated investment in short-term bonds',0)
xlabel('Date of Maturity');ylabel('(Expected) Total return');grid on;

%COUNTRY Y 
%COUNTRY Y with following values
load ValY.mat
r=ValY(1);
kappa=ValY(2);
theta=ValY(3);
sigma=ValY(4);
lambda=ValY(5);
gamma=((kappa+lambda)^2+2*(sigma^2))^0.5;
%Long term value of the yield%
Rl=(2*kappa*theta)/(gamma+kappa+lambda);
%Critical value, which determines whether or not the term structure is humped
Rg=(kappa*theta)/(kappa+lambda);
phi1=gamma;
phi2=(kappa+lambda+gamma)*0.5;
phi3=(2*kappa*theta)/(sigma^2);

%This part calculates the term structure of default-free discount bonds accoding to the paper of CIRII
%Yield converges to the spot rate as maturity nears: Tau==Null
Rrt(1)=r;
i=1;    
for Tau=0.01:0.01:lasttime-t
    i=i+1;
    N=phi2*(exp(phi1*(Tau))-1)+phi1;    
    At=(phi1*exp(phi2*(Tau))/N)^phi3;
    Bt=(exp(phi1*(Tau))-1)/N;
    if At~=0
        Rrt(i)=(-log(At)+r*Bt)/(Tau);
        RTRl(i-1)=exp(Rrt(i)*Tau)-1;
    else
        Rrt(i)=NaN;
        RTRl(i-1)=NaN;
    end
end

%Plotting the total return when at time t=0 a zero bond with maturity Tau=[0.01:0.01:lasttime-t] is chosen
visu6=figure;
Values=[0.01+t:0.01:lasttime;RTRl]; 
plot(Values(1,:),Values(2,:));hold on;axis tight

%These values can be chosen constant, cause we only investigate bonds with maturity=0.01
N=phi2*(exp(phi1*(0.01))-1)+phi1;    
At=(phi1*exp(phi2*(0.01))/N)^phi3;
Bt=(exp(phi1*(0.01))-1)/N;

%This part calculates the expectation of the total return when reinvesting every period
%The expected total return for a zero bond with maturity=0.01 at current time t
%can be calculated as follows:
RTR(1)=exp(Rrt(2)*0.01)-1;
i=1;
for s=0.01+t:0.01:lasttime-0.01
    i=i+1;    
    c=(2.*kappa)./(sigma.^2.*(1-exp(-kappa.*(s-t))));
    u=c.*r.*exp(-kappa.*(s-t));
    q=(2.*kappa.*theta)./(sigma.^2)-1;
    RTR(i)=(1/At)*(c/(c-Bt))^(q+1)*exp((u*Bt)/(c-Bt))-1;
end

%Calculation of the effective yield when reinvesting every period
i=1;
for s=0.01+t:0.01:lasttime    
    QMRTR(i)=1;
    for j=1:i
        QMRTR(i)=QMRTR(i)*(RTR(j)+1);
    end
    QMRTR(i)=QMRTR(i)-1;
    i=i+1;
end

%Plotting of the total return when reinvesting every period
plot(Values(1,:),QMRTR,'r');hold off;axis tight
title('Country Y: Long-term bonds vs. repeated reinvestment');
legend('Total return of long-term bond','Expected total return of repeated investment in short-term bonds',0)
xlabel('Date of Maturity');ylabel('(Expected) Total return');grid on;

%SAVING OF PLOT with counting number i
i=1;
saveas(visu5,['liqpref',mat2str(i),'-1.eps']);
saveas(visu6,['liqpref',mat2str(i),'-2.eps']);
saveas(visu5,['liqpref',mat2str(i),'-1.fig']);
saveas(visu6,['liqpref',mat2str(i),'-2.fig']);

clear all;
